# encoding:utf-8
from enum import Enum


class TaskType(str, Enum):
    """
    任务类型枚举
    """
    # 商品基础信息
    SKU_INFO = 'sku_info'
    # 商品价格
    SKU_PRICE = 'sku_price'
    # 商品基础信息及价格等详细信息
    SKU_FULL = 'sku_full'
    # 商品主详图
    SKU_IMAGE = 'sku_image'
    # 流式采集商品编码（如编码）
    STREAM_SKU_ID = 'stream_sku_id'
    # 流式采集商品价格信息
    STREAM_SKU_PRICE = 'stream_sku_price'
    # 流式采集商品基础信息及价格等详细信息
    STREAM_SKU_FULL = 'stream_sku_full'
    # 流式采集商品主详图
    STREAM_SKU_IMAGE = 'stream_sku_image'
    # 流式搜索同款商品
    STREAM_SKU_SAME = 'stream_sku_same'
    # 评论数量
    STREAM_SKU_COMMENT = 'stream_sku_comment'

    @staticmethod
    def to_list():
        enum_list = list()
        for tuple_item in TaskType.__members__.items():
            enum_list.append({'id': tuple_item[0], 'name': tuple_item[1].value})
        return enum_list


class CrawlJobStatus(str, Enum):
    """
    采集任务状态枚举
    """

    CREATED = "已创建"
    RUNNING = "采集中"
    CANCELED = "已取消"
    ERROR = "启动失败"
    CRAWLING = "采集中"
    CRAWLED = "采集完成"


class CommonStatus(str, Enum):
    """
    通用状态枚举
    """
    # 启用
    ON = '已启用'
    # 停用
    OFF = '已停用'


class StreamSpiderGroupType(str, Enum):
    """
    流采爬虫组枚举
    """
    # 商品信息
    skuInfo = "sku_info"
    # 商品评论
    skuComment = "sku_comment"
    # 商品图片
    skuImage = "sku_image"
    # 商品价格
    skuPrice = "sku_price"
    # 电商比价
    priceCompare = "price_compare"

    @staticmethod
    def to_list():
        enum_list = list()
        for tuple_item in StreamSpiderGroupType.__members__.items():
            enum_list.append({'id': tuple_item[0], 'groupCode': tuple_item[1].value})
        return enum_list


class TaskMode(str, Enum):
    """
    爬虫任务类型
    """
    # 单个
    single = "single"
    # 批次
    batch = "batch"



# encoding: utf-8

from typing import Optional, List
from zcbot_crawl_core.client.mongo_client import Mongo
from zcbot_crawl_core.model.enums import CommonStatus
from zcbot_crawl_core.model.entity import StreamSpider


def get_spider_group_list(group_code: str, status: str = CommonStatus.ON.name):
    _query = {}
    if group_code:
        _query['groupCode'] = group_code

    _query['status'] = status

    return Mongo().list(collection="zcbot_stream_spider_group", query=_query)


def get_spider_by_spider_id(spider_id: str, status: str = CommonStatus.ON.name) -> Optional[StreamSpider]:
    _query = {
        "_id": spider_id,
        "status": status
    }
    result = Mongo().get(collection="zcbot_stream_spider", query=_query)

    if result:
        return StreamSpider(**result)

    return None


def get_spider_list(spider_ids: List[str], status: str = CommonStatus.ON.name) -> List[StreamSpider]:
    _query = {}
    _query['_id'] = {"$in": spider_ids}
    _query['status'] = status

    temp = []
    result = Mongo().list(collection="zcbot_stream_spider", query=_query)
    if result:
        temp = [StreamSpider(**x) for x in result]

    return temp



# -*- coding: utf-8 -*-
from typing import List
from ..client.mongo_client import Mongo


# 获取支持网站平台
def get_platforms_by_group(group_code: str, enable: int = None):
    _query = {}
    if group_code:
        _query["groupCode"] = group_code
    if enable is not None:
        _query["enable"] = enable

    return Mongo().list('zcbot_batch_spider_group', query=_query)


def get_platforms_by_group_code(group_code: str, enable):
    _query = {
        "groupCode": group_code
    }
    _fields = {
        "spiders": 0,
        "_id": 0
    }
    return Mongo().aggregate('zcbot_platforms', [
        {
            '$lookup': {
                'from': 'zcbot_batch_spider_group',
                'localField': '_id',
                'foreignField': 'platCode',
                'as': 'spider'
            }
        },
        {
            '$match': {'spider.groupCode': group_code}
        },
        {
            '$project': {'spider': 0}
        },
        {'$sort': {'sort': 1}}
    ])


# 根据爬虫组编号，获取可选的爬虫清单
def get_spider_group(group_code: str = None, plat_code: str = None, enable: int = None):
    _query = {}
    if group_code:
        _query["groupCode"] = group_code
    if plat_code:
        _query["platCode"] = plat_code
    if enable is not None:
        _query["enable"] = enable

    return Mongo().get('zcbot_batch_spider_group', query=_query)


# 根据爬虫组编号，获取爬虫组清单
def get_spider_group_list(group_code: str = None, plat_codes: List[str] = None, enable: int = None):
    _query = {}
    if group_code:
        _query["groupCode"] = group_code
    if plat_codes and len(plat_codes):
        _query["platCode"] = {"$in": plat_codes}
    if enable is not None:
        _query["enable"] = enable

    return Mongo().list('zcbot_batch_spider_group', query=_query)



# -*- coding: utf-8 -*-
import pytz
from pymongo import MongoClient

from ..util import cfg
from ..util.exceptions import NoConfigException
from ..util.decator import singleton


@singleton
class Mongo(object):
    """
    Mongo基础客户端简易封装
    """

    def __init__(self, mongo_uri: str = None, mongo_db: str = None, *args, **kwargs):
        self.mongo_uri = mongo_uri or cfg.get('ZCBOT_CORE_MONGO_URL')
        self.mongo_db = mongo_db or cfg.get('ZCBOT_CORE_MONGO_DB')
        if not self.mongo_uri:
            raise NoConfigException('mongodb uri not config!')
        if not self.mongo_db:
            raise NoConfigException('mongodb database not config!')

        self.client = MongoClient(self.mongo_uri, tz_aware=True, tzinfo=pytz.timezone('Asia/Shanghai'))
        self.db = self.client[self.mongo_db]

    # 获取集合
    def get_collection(self, coll):
        return self.db.get_collection(coll)

    # 查询对象
    def get(self, collection, query={}):
        result = self.db[collection].find_one(query)
        return result

    # 统计数量
    def count(self, collection, query={}):
        return self.db[collection].count(query)

    # 查询列表
    def list(self, collection, query={}, fields=None, sort=[]):
        if fields:
            cursor = self.db[collection].find(query, fields, sort=sort)
        else:
            cursor = self.db[collection].find(query, sort=sort)
        return list(cursor)

    # 查询去重列表
    def distinct(self, collection, dist_key, query={}, fields=None):
        return self.db[collection].find(query, fields).distinct(dist_key)

    # 聚合查询
    def aggregate(self, collection, pipeline=[]):
        cursor = self.db[collection].aggregate(pipeline, session=None, allowDiskUse=True)
        return list(cursor)

    # 查询分页列表
    def list_with_page(self, collection, query={}, page_size=10000, fields=None):
        rows = list()
        total = self.db[collection].count(query)
        if total > 0 and page_size > 0:
            total_page = round(len(total) / page_size)
            for page in range(0, total_page):
                if fields:
                    cursor = self.db[collection].find(query, fields).skip(page_size * page).limit(page)
                else:
                    cursor = self.db[collection].find(query).skip(page_size * page).limit(page)
                curr_batch = list(cursor)
                if curr_batch:
                    rows.append(curr_batch)
        return rows

    # 插入或更新
    def insert_or_update(self, collection, data, id_key='_id'):
        return self.db[collection].update({id_key: data[id_key]}, {'$set': data}, upsert=True)

    # 更新
    def update(self, collection, filter, data, multi=False):
        return self.db[collection].update(filter, {'$set': data}, multi=multi)

    # 以主键更新
    def update_by_pk(self, collection, pk_val, data):
        return self.db[collection].update({'_id': pk_val}, {'$set': data}, multi=False)

    # 批量更新
    def batch_update(self, collection, filter, datas, multi=False):
        return self.db[collection].update(filter, datas, multi=multi)

    # 更新
    def delete(self, collection, filter):
        return self.db[collection].delete_many(filter)

    # 插入或更新
    def bulk_write(self, collection, bulk_list):
        if bulk_list:
            return self.db[collection].bulk_write(bulk_list, ordered=False, bypass_document_validation=True)

    # 关闭链接
    def close(self):
        self.client.close()

    # 销毁关闭链接
    def __del__(self):
        if self.client:
            self.client.close()



from typing import List
from pydantic import BaseModel


class BatchSpider(BaseModel):
    """
    Portainer爬虫
    """
    # 爬虫id(主键)
    spiderId: str = None
    # 容器镜像标签
    dockerImage: str = None
    # 运行启动参数模板
    param: str = None
    # 任务模式（批量multi、单个single）
    taskMode: str = None
    # 批次请求大小（批量模式有效）
    batchSize: int = 1
    # 可运行节点编号列表
    nodes: List[str] = None
    # 电商平台编码
    platCode: str = None
    # 电商平台名称
    platName: str = None
    # 任务类型
    taskType: str = None
    # 任务类型描述
    taskTypeText: str = None
    # 备注
    remark: str = None
    # 支持链接规则编码集合
    patterns: List[str] = None
    # 运行环境变量集合
    env: List[str] = None
    # 状态
    status: str = None
    statusText: str = None


class SupportPlatform(BaseModel):
    """
    支持平台
    """
    # 【输入】平台编码
    platCodes: List[str] = None
    # 【输入】
    groupCode: str = None


class Spider(BaseModel):
    spiderId: str = None
    # 平台名称
    platCode: str = None
    # 任务类型
    taskType: str = None
    # 任务类型(显示用)
    taskTypeText: str = None
    # 是否默认可点击
    defaultChecked: int = None
    # 备份
    remark: str = None
    # 状态
    status: str = None
    statusText: str = None


class BatchSpiderGroup(BaseModel):
    # 爬虫组(主键)
    groupId: str = None
    # 爬虫信息
    spiders: List[Spider] = []
    # 平台编码
    platCode: str = None
    # 平台名称
    platName: str = None
    # 组编码
    groupCode: str = None
    # 组名称
    groupName: str = None
    # 标题
    title: str = None
    # 状态
    status: str = None
    statusText: str = None
    # 商品状态选择
    skuStatusElect: str = None


class PortainerNode(BaseModel):
    """
    Portainer平台节点
    """
    # 端点序列号 全局唯一（自定义，不可直接使用节点Id字段）
    nodeId: str = None
    # 端点名称
    nodeName: str = None
    # 备注
    remark: str = None

    # api base url
    apiBaseUrl: str = None
    # api token
    apiToken: str = None
    # /api/endpoints中的Id字段
    endpointId: str = None

    # 状态
    status: str = None
    statusText: str = None


class StreamSpider(BaseModel):
    """
    流程爬虫
    """
    # 爬虫项目名称
    botName: str = None
    # redis队列
    redisKey: str = None
    # 爬虫id(主键)
    spiderId: str = None
    # 爬虫名称
    spiderName: str = None
    # 爬虫信号
    signals: List[str] = []
    # 任务模式（批量batch、单个single）
    taskMode: str = None
    # 批次请求大小（批量模式有效）
    batchSize: int = 1
    # 电商平台编码
    platCode: str = None
    # 电商平台名称
    platName: str = None
    # 任务类型
    taskType: str = None
    # 任务类型描述
    taskTypeText: str = None
    # 备注
    remark: str = None
    # 支持链接规则编码集合
    patterns: List[str] = None
    # 状态
    status: str = None
    # 状态
    statusText: str = None
    # skuStatusElect
    skuStatusElect: str = None


class StreamSpiderModel(BaseModel):
    """
    流采爬虫配置表
    """
    # sn,id,爬虫id
    # _id命名规则：groupCode:spiderProjectName:spiderName
    # eg: sku_full:stream_sku_jd:sku_info_pc_v1
    _id: str = None
    # 爬虫组编码
    groupCode: str = None
    # 爬虫组名称
    groupName: str = None
    # 平台编码
    platCode: str = None
    # 平台名称
    platName: str = None
    # 爬虫项目名称
    botName: str = None
    # 爬虫id
    spiderId: str = None
    # 爬虫名称
    spiderName: str = None
    # 爬虫状态
    status: str = None
    # 爬虫状态中文
    statusText: str = None
    # redis队列
    redisKey: str = None
    # 任务模式 单条/批量 single/batch
    taskMode: str = None
    # 批次大小 1/60
    batchSize: int = None
    # 备注
    remark: str = None


class StreamSpiderForGroup(BaseModel):
    # 爬虫id
    spiderId: str = None
    # 平台编码
    platCode: str = None
    # 爬虫状态
    status: str = None
    # 爬虫状态
    statusText: str = None
    # 备注
    remark: str = None


class StreamSpiderGroup(BaseModel):
    # 爬虫组
    groupCode: str = None
    # 爬虫组名称
    groupName: str = None
    # 爬虫组状态
    status: str = None
    # 爬虫组状态
    statusText: str = None
    # 组内爬虫清单
    spiders: List[StreamSpiderForGroup]



# -*- coding: utf-8 -*-

from typing import List, Dict, Optional
from ..model.enums import CommonStatus, StreamSpiderGroupType
from ..model.entity import StreamSpiderGroup, StreamSpider
from ..dao import stream_spider_group as stream_spider_group_dao


def get_all_stream_spider_groups() -> List[Dict[str, str]]:
    """
    获取所有的爬虫组
    """

    return StreamSpiderGroupType.to_list()


def get_groups(group_code: str, status: str = CommonStatus.ON.name) -> List[StreamSpiderGroup]:
    """
    根据一个爬虫组编码，获取里面所有的爬虫
    """
    rows = stream_spider_group_dao.get_spider_group_list(group_code=group_code, status=status)
    rows = [StreamSpiderGroup(**x) for x in rows]

    return rows


def get_spider(spider_id: str, status: str = CommonStatus.ON.name) -> Optional[StreamSpider]:
    """
    获取爬虫详情
    """

    stream_spider = stream_spider_group_dao.get_spider_by_spider_id(spider_id=spider_id, status=status)

    return stream_spider


def get_spiders(group_code: str, plat_code: str, status: str = CommonStatus.ON.name) -> List[StreamSpider]:
    """
    通过爬虫组编号以及平台，获取对应的爬虫，返回空列表或者是对应列表
    """
    stream_spider_groups = get_groups(group_code=group_code, status=status)
    plat_spider_ids = []
    for spider_group in stream_spider_groups:
        for spider in spider_group.spiders:
            if spider.platCode == plat_code and spider.status == status:
                plat_spider_ids.append(spider.spiderId)

    stream_spiders = stream_spider_group_dao.get_spider_list(spider_ids=plat_spider_ids, status=status)

    return stream_spiders





# -*- coding: UTF-8 -*-


# 采集任务队列
def get_task_queue_key(batch_id: str, spider_id: str):
    return f'zcbot:{batch_id}:{spider_id}'


# 采集任务队列
def get_task_queue_key_for_remove(batch_id: str):
    return f'zcbot:{batch_id}:*'


# 采集重试任务源数据映射
def get_retry_request_source_key(request_id: str):
    return f'zcbot:request-source:{request_id}'


# 测试
if __name__ == '__main__':
    pass



"""
Time Utilities

fast approaches to commonly used time/date related functions
"""
import datetime
import time
from typing import Optional
from dateutil import tz


def now() -> datetime.datetime:
    """
    get datetime instance of time of now
    :return: time of now
    """
    return datetime.datetime.now(tz.gettz('Asia/Shanghai'))


def __get_t(t: Optional[datetime.datetime] = None) -> datetime.datetime:
    """
    get datetime instance
    :param t: optional datetime instance
    :return: datetime instance
    """
    return t if isinstance(t, datetime.datetime) else now()


def to_str(t: Optional[datetime.datetime] = None,
           fmt: str = '%Y-%m-%d %H:%M:%S.%f') -> str:
    """
    get string formatted time
    :param t: optional datetime instance
    :param fmt: string format
    :return:
    """
    return __get_t(t).strftime(fmt)


def to_seconds(t: Optional[datetime.datetime] = None) -> int:
    """
    datetime to seconds
    :param t: optional datetime instance
    :return: timestamp in seconds
    """
    return int(__get_t(t).timestamp())


def to_milliseconds(t: Optional[datetime.datetime] = None) -> int:
    """
    datetime to milliseconds
    :param t: datetime instance
    :return: timestamp in seconds
    """
    return int(__get_t(t).timestamp() * 10 ** 3)


def to_microseconds(t: Optional[datetime.datetime] = None) -> int:
    """
    datetime to microseconds
    :param t: datetime instance
    :return: timestamp in seconds
    """
    return int(__get_t(t).timestamp() * 10 ** 6)


def get_dt(start_t: datetime.datetime,
           end_t: Optional[datetime.datetime] = None) -> datetime.timedelta:
    """
    get delta time
    :param start_t: start time
    :param end_t: end time
    :return: timedelta instance
    """
    return __get_t(end_t) - start_t


def to_seconds_dt(dt: datetime.timedelta) -> int:
    """
    delta time to seconds
    :param dt: timedelta instance
    :return: seconds elapsed
    """
    return int(dt.total_seconds())


def to_milliseconds_dt(dt: datetime.timedelta) -> int:
    """
    delta time to milliseconds
    :param dt: timedelta instance
    :return: milliseconds elapsed
    """
    return int(dt.total_seconds() * 10 ** 3)


def to_microseconds_dt(dt: datetime.timedelta) -> int:
    """
    delta time to microseconds
    :param dt: timedelta instance
    :return: microseconds elapsed
    """
    return int(dt.total_seconds() * 10 ** 6)


def parse_timestamp(time_str, tz_str='Asia/Shanghai'):
    """
    将时间戳解析成本地时间(自动截断毫秒)
    :param time_str:
    :param tz_str:
    :return:
    """
    if time_str:
        # 1576839034000
        if len(str(time_str)) > 10:
            # 截取掉毫秒
            time_str = str(time_str)[0:10]
        return datetime.datetime.fromtimestamp(int(time_str)).astimezone(tz.gettz(tz_str))


def time_to_batch_id(dt, delta_hour=0, delta_day=0, err=''):
    """
    # 将采购时间转换为批次编号(delta为0时，时间应为iso时间)，delta为正则加为负则减
    """
    try:
        if dt and isinstance(dt, datetime.datetime):
            if delta_hour or delta_day:
                time_delta = datetime.timedelta(hours=int(delta_hour), days=int(delta_day))
                dt = dt + time_delta
            return int(dt.strftime("%Y%m%d"))
    except ValueError:
        return err


def current_timestamp13():
    """
    13位当前时间时间戳（毫秒级时间戳）
    :return:
    """
    return int(round(time.time() * 1000))


def current_timestamp10():
    """
    10位当前时间时间戳（秒级时间戳）
    :return:
    """
    return int(time.time())


if __name__ == '__main__':
    print(parse_timestamp(str(int(now().timestamp()))))
